# a very simple tcp simulation to start with
# run the simulation and view in nam.
# Now change the queue size of r1 and r2 to 10. (Uncomment the 2 lines.)
# Do you see pkt drops in nam?
# Now change the window size of the tcp connection to 10. Do you see drops?


#
# Create a simple four node topology:
#
#        s1                 s2
#         \                 /
# 10Mb,2ms \  1.5Mb,20ms   / 10Mb,4ms
#           r1 --------- r2


set ns [new Simulator]

#Define different colors for data flows (for NAM)
$ns color 0 Red
$ns color 2 Blue

set f [open 2-out.tr w]
$ns trace-all $f  

set nf [open 2-simple.nam w]
$ns namtrace-all $nf

set f0 [open 2-band.tr w]

set node_(s1) [$ns node]
set node_(s2) [$ns node]
set node_(r1) [$ns node]
set node_(r2) [$ns node]

$ns duplex-link $node_(s1) $node_(r1) 10Mb 2ms DropTail 
$ns duplex-link $node_(s2) $node_(r2) 10Mb 4ms DropTail 
$ns duplex-link $node_(r1) $node_(r2) 1.5Mb 10ms RED 
#$ns duplex-link $node_(r1) $node_(r2) 1.5Mb 10ms DropTail
$ns queue-limit $node_(r1) $node_(r2) 10
$ns queue-limit $node_(r2) $node_(r1) 10


$ns duplex-link-op $node_(s1) $node_(r1) orient right-down 
$ns duplex-link-op $node_(s2) $node_(r2) orient left-down
$ns duplex-link-op $node_(r1) $node_(r2) orient right
$ns duplex-link-op $node_(r1) $node_(r2) queuePos 0.5
$ns duplex-link-op $node_(r2) $node_(r1) queuePos 0.5



set tcp [new Agent/TCP/Reno]
set sink [new Agent/TCPSink]
#set sink [new Agent/LossMonitor]
#$tcp set window_ 10
$tcp set class_ 0

$ns attach-agent $node_(s1) $tcp
$ns attach-agent $node_(s2) $sink
$ns connect $tcp $sink

set ftp [new Application/FTP]
$ftp attach-agent $tcp
$ns at 0.1 "$ftp start"

$ns at 60 "finish"
#$ns at 5 "finish"

proc record {} {
        global sink f0
        #Get an instance of the simulator
        set ns [Simulator instance]
        #Set the time after which the procedure should be called again
        set time 0.5
        #How many bytes have been received by the traffic sinks?
        set bw0 [$sink set bytes_]
        #Get the current time
        set now [$ns now]
        #Calculate the bandwidth (in MBit/s) and write it to the files
        puts $f0 "$now [expr $bw0/$time*8/1000000]"
        #Reset the bytes_ values on the traffic sinks
        $sink set bytes_ 0

        #Re-schedule the procedure
        $ns at [expr $now+$time] "record"
}
$ns at 0 "record"

proc finish {} {
    global ns f nf f0
    $ns flush-trace
#    close $f
    close $nf
    #close $f0
    puts "running nam..."
    exec xgraph 2-band.tr -geometry 800x400 &
    exec nam 2-simple.nam &
    exit 0
}
$ns run
