# a very simple tcp simulation to start with
# run the simulation and view in nam.
# Now change the queue size of r1 and r2 to 10. (Uncomment the 2 lines.)
# Do you see pkt drops in nam?
# Now change the window size of the tcp connection to 10. Do you see drops?


#
# Create a simple four node topology:
#
#        s1                 s2
#         \                 /
# 10Mb,2ms \  1.5Mb,20ms   / 10Mb,4ms
#           r1 --------- r2


set ns [new Simulator]

#Define different colors for data flows (for NAM)
$ns color 0 Red
$ns color 1 Green
$ns color 2 Blue
$ns color 3 Black
$ns color 4 Orange
$ns color 5 Yellow 

set f [open 3-out.tr w]
$ns trace-all $f  

set nf [open 3-simple.nam w]
$ns namtrace-all $nf

set f0 [open 4-band-ftp1.tr w]
set f1 [open 4-band-ftp2.tr w]
set f2 [open 4-band-video.tr w]



set node_(s1) [$ns node]
set node_(s3) [$ns node]
set node_(r1) [$ns node]
set node_(r2) [$ns node]
set node_(s2) [$ns node]
set node_(s4) [$ns node]

$ns duplex-link $node_(s1) $node_(r1) 10Mb 2ms DropTail 
$ns duplex-link $node_(s2) $node_(r1) 10Mb 3ms DropTail 
$ns simplex-link $node_(r1) $node_(r2) 1.5Mb 20ms CBQ
$ns simplex-link $node_(r2) $node_(r1) 1.5Mb 10ms DropTail
$ns duplex-link $node_(s3) $node_(r2) 10Mb 4ms DropTail
$ns duplex-link $node_(s4) $node_(r2) 10Mb 5ms DropTail

# CBQ-Template

set cbqlink [$ns link $node_(r1) $node_(r2)]
set topClass [new CBQClass]
$topClass setparams none 0 1.0 auto 8 1 0

set UDPgold [new CBQClass]
$UDPgold setparams $topClass true 0.6 auto 1 1 0

set FTPsilver [new CBQClass]
$FTPsilver setparams $topClass true 0.39 auto 1 1 0

set Rest [new CBQClass]
$Rest setparams $topClass true 0.01 auto 1 1 0

set gq [new Queue/SFQ]
$gq set queue-limit 10
set sq [new Queue/SFQ]
$sq set queue-limit 10
set rq [new Queue/SFQ]
$rq set queue-limit 10

$UDPgold install-queue $gq
$FTPsilver install-queue $sq
$Rest install-queue $rq

$cbqlink insert $topClass
$cbqlink insert $UDPgold
$cbqlink insert $FTPsilver
$cbqlink insert $Rest

$cbqlink bind $UDPgold 3
$cbqlink bind $FTPsilver 1
$cbqlink bind $FTPsilver 0
$cbqlink bind $Rest 4
$cbqlink bind $Rest 5





$ns queue-limit $node_(r1) $node_(r2) 10
$ns queue-limit $node_(r2) $node_(r1) 10


$ns duplex-link-op $node_(s1) $node_(r1) orient right-down 
$ns duplex-link-op $node_(s2) $node_(r1) orient right-up
$ns duplex-link-op $node_(r1) $node_(r2) orient right
$ns duplex-link-op $node_(s3) $node_(r2) orient left-down
$ns duplex-link-op $node_(s4) $node_(r2) orient left-up
$ns duplex-link-op $node_(r1) $node_(r2) queuePos 0.5
$ns duplex-link-op $node_(r2) $node_(r1) queuePos 0.5



set tcp1 [new Agent/TCP/Reno]
set tcp2 [new Agent/TCP/Reno]
set udp1 [new Agent/UDP]
set udp2a [new Agent/UDP]
set udp2b [new Agent/UDP]
set sink1 [new Agent/TCPSink]
set sink2 [new Agent/TCPSink]
#set sink4a [new Agent/Null]
set sink3 [new Agent/Null]
set sink4b [new Agent/Null]
set sink4a [new Agent/LossMonitor]
#$tcp set window_ 10
$tcp1 set class_ 0
$tcp2 set class_ 1
$udp1 set class_ 3
$udp2a set class_ 4
$udp2b set class_ 5

$ns attach-agent $node_(s1) $tcp1
$ns attach-agent $node_(s2) $tcp2
$ns attach-agent $node_(s1) $udp1
$ns attach-agent $node_(s2) $udp2a
$ns attach-agent $node_(s2) $udp2b
$ns attach-agent $node_(s3) $sink1
$ns attach-agent $node_(s3) $sink2
$ns attach-agent $node_(s4) $sink4a
$ns attach-agent $node_(s3) $sink3
$ns attach-agent $node_(s4) $sink4b
$ns connect $tcp1 $sink1
$ns connect $tcp2 $sink2
$ns connect $udp1 $sink4a
$ns connect $udp2a $sink3
$ns connect $udp2b $sink4b

set ftp1 [new Application/FTP]
set ftp2 [new Application/FTP]
set cbr1 [new Application/Traffic/CBR]
set cbr2a [new Application/Traffic/CBR]
set cbr2b [new Application/Traffic/CBR]

$cbr1 set packet_size_ 1000
$cbr1 set rate_ 1mb

$cbr2a set packet_size_ 367
$cbr2a set rate_ 0.35mb

$cbr2b set packet_size_ 430
$cbr2b set rate_ 0.56mb

$ftp1 attach-agent $tcp1
$ftp2 attach-agent $tcp2
$cbr1 attach-agent $udp1
$cbr2a attach-agent $udp2a
$cbr2b attach-agent $udp2b

$ns at 0.5 "$ftp1 send 500000000"
$ns at 0.8 "$ftp2 send 500000000"
$ns at 0 "$cbr1 start"
$ns at 0 "$cbr2a start"
$ns at 0 "$cbr2b start"

$ns at 60 "finish"
#$ns at 5 "finish"

proc record {} {
        global sink1 sink2 sink4a f0 f1 f2
        #Get an instance of the simulator
        set ns [Simulator instance]
        #Set the time after which the procedure should be called again
        set time 0.5
        #How many bytes have been received by the traffic sinks?
        set bw0 [$sink1 set bytes_]
        set bw1 [$sink2 set bytes_]
        set bw2 [$sink4a set bytes_]
        #Get the current time
        set now [$ns now]
        #Calculate the bandwidth (in MBit/s) and write it to the files
        puts $f0 "$now [expr $bw0/$time*8/1000000]"
	puts $f1 "$now [expr $bw1/$time*8/1000000]"
        puts $f2 "$now [expr $bw2/$time*8/1000000]"
        #Reset the bytes_ values on the traffic sinks
        $sink1 set bytes_ 0
	$sink2 set bytes_ 0
	$sink4a set bytes_ 0

        #Re-schedule the procedure
        $ns at [expr $now+$time] "record"
}
$ns at 0 "record"

proc finish {} {
    global ns f nf f0 f1
    $ns flush-trace
#    close $f
    close $nf
    close $f0
    close $f1
    puts "running nam..."
    exec xgraph 4-band-ftp1.tr -geometry 800x400 &
    exec xgraph 4-band-ftp2.tr -geometry 800x400 &
    exec xgraph 4-band-video.tr -geometry 800x400 &
    exec nam 3-simple.nam &
    exit 0
}
$ns run
